(*
    ===========================================================================
    AboutBox
    ---------------------------------------------------------------------------
    2003/12/02  C
    2005/02/19  C
    ===========================================================================
*)
unit Info;

interface

uses
    Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms,
    Buttons, ExtCtrls, StdCtrls, ShellAPi, ShlObj, ActiveX, ComObj;

type
  TInfoForm = class(TForm)
    AppIcon: TImage;
    LblExeName: TLabel;
    lblComment: TLabel;
    LblOsVersion: TLabel;
    LblMemPhys: TLabel;
    LblExeVersion: TLabel;
    lblURL: TLabel;
    lblMail: TLabel;
    procedure FormCreate(Sender: TObject);
    procedure lblURLClick(Sender: TObject);
    procedure lblMailClick(Sender: TObject);
    procedure FormKeyDown(Sender: TObject; var Key: Word;
      Shift: TShiftState);
  private

  public

  end;

procedure OpenInfoForm();
procedure SimpleShellExecute(const FileName: string);
function GetOsInfo(): string;
procedure GetMemoryStatus(var Phys, Page, Virt: string);
function LoadVersionInfo(const ExePath: string): string;
function GetStartUpFolderPath(): string;
procedure CreateStartup();

implementation

{$R *.dfm}

//-----------------------------------------------------------------------------
//  \
procedure   OpenInfoForm();
var
    InfoForm: TInfoForm;
begin
    InfoForm := TInfoForm.Create(Application);
    try
        InfoForm.ShowModal();
    finally
        InfoForm.Free();
    end;
end;

//=============================================================================
//  tH[Cxgnh
//=============================================================================

//-----------------------------------------------------------------------------
//  tH[쐬
procedure TInfoForm.FormCreate(Sender: TObject);
var
    S1, S2, S3: string;
begin
    Self.Caption := Application.Title + ' - ' + Self.Name;

    lblExeName.Caption := ExtractFileName(ParamStr(0));

    AppIcon.Picture.Icon := Application.Icon;
    LblOsVersion.Caption := StringReplace(GetOsInfo(), 'Microsoft ', '', []);

    GetMemoryStatus(S1, S2, S3);
    LblMemPhys.Caption := S1;

    LblExeVersion.Caption := Format('ver.%s', [LoadVersionInfo(ParamStr(0))]);

    Self.AutoSize := True;
end;

//-----------------------------------------------------------------------------
//  z[y[WN
procedure TInfoForm.lblURLClick(Sender: TObject);
begin
    SimpleShellExecute(lblURL.Caption);
end;

//-----------------------------------------------------------------------------
//  [[N
procedure TInfoForm.lblMailClick(Sender: TObject);
begin
    SimpleShellExecute(Format('MailTo:%s?subject=from [%s] user', [
        lblMail.Caption, ExtractFileName(Application.ExeName)])
    );
end;

//-----------------------------------------------------------------------------
//  ̃L[ꂽ
procedure TInfoForm.FormKeyDown(Sender: TObject; var Key: Word;
  Shift: TShiftState);
begin
    Self.Close();
end;

//=============================================================================
//  [eBeB֐
//=============================================================================

//-----------------------------------------------------------------------------
//  ShellExecutebp[
procedure SimpleShellExecute(const FileName: string);
begin
    try
        ShellExecute(Application.MainForm.Handle,
                     nil,
                     PChar(FileName),
                     nil,
                     PChar(ExtractFilePath(FileName)),
                     SW_SHOW);
    except
        ;
    end;
end;

//=============================================================================
//  OS̃o[W
//=============================================================================
type
    TOSVersionInfoEx = record
        dwOSVersionInfoSize : DWORD;
        dwMajorVersion      : DWORD;
        dwMinorVersion      : DWORD;
        dwBuildNumber       : DWORD;
        dwPlatformId        : DWORD;
        szCSDVersion        : array[0..127] of AnsiChar;
        wServicePackMajor   : WORD;
        wServicePackMinor   : WORD;
        wSuiteMask          : WORD;
        wProductType        : BYTE;
        wReserved           : BYTE;
    end;

// Windows.pas
function GetVersionEx(var lpVersionInformation: TOSVersionInfoEx): BOOL; stdcall; external kernel32 name 'GetVersionExA';

// http://www.geocities.jp/ht_deko/ft0203.html
// http://plaza5.mbn.or.jp/~heropa/vb25.htm
const
    VER_NT_WORKSTATION       = 1;
    VER_NT_DOMAIN_CONTROLLER = 2;
    VER_NT_SERVER            = 3;
    VER_SUITE_SMALLBUSINESS            = $0001;
    VER_SUITE_ENTERPRISE               = $0002;
    VER_SUITE_BACKOFFICE               = $0004;
    VER_SUITE_TERMINAL                 = $0010;
    VER_SUITE_SMALLBUSINESS_RESTRICTED = $0020;
    VER_SUITE_DATACENTER               = $0080;
    VER_SUITE_PERSONAL                 = $0200;
    VER_SUITE_BLADE                    = $0400;


(* 蔲Ńo[W擾 *)
function    GetOsInfoEasy(): string;

    function	PlatformId(ID: Cardinal): string;
    begin
        case ID of
            VER_PLATFORM_WIN32s			: Result := 'Windows 3.1';
            VER_PLATFORM_WIN32_WINDOWS	: Result := 'Windows 95';
            VER_PLATFORM_WIN32_NT		: Result := 'Windows NT';
        end;
    end;

var
    osi: TOSVersionInfo;
begin
    FillChar(osi, SizeOf(TOSVersionInfo), #0);
    osi.dwOSVersionInfoSize := SizeOf(TOSVersionInfo);
    Windows.GetVersionEx(osi);
    Result := Format('%s Version %d.%d (Build %d : %s)',
            [PlatformId(osi.dwPlatformId),
            osi.dwMajorVersion,
            osi.dwMinorVersion,
            osi.dwBuildNumber,
            osi.szCSDVersion
        ]);
end;

(* {CŃo[W擾 *)
function    GetOsInfoHard(): string;
var
    osvi:TOSVersionInfoEx;
    res : Boolean;
    Key: HKEY;
    szProductType: string;
    dwBufLen, dwTemp: DWORD;
    lRet: integer;
begin
    Result := 'Unknown';

    // OSVERSIONINFOEX  GetVersionEx Ă݂ 
    // s OSVERSIONINFO łĂ݂ 

    // 
    FillChar(osvi, SizeOf(TOSVersionInfoEx), #0);

    // \̃TCYZbg
    osvi.dwOSVersionInfoSize := SizeOf(TOSVersionInfoEx);

    // Ă݂
    res := GetVersionEx(osvi);

    // s
    if not res then
    begin
        // ЂƂ̍\̂
        osvi.dwOSVersionInfoSize := SizeOf(TOSVersionInfo);
        // ߂Ȃ炠߂
        if not GetVersionEx(osvi) then Exit;
    end;

    // dwPlatformId ŌɂȂقǏꍇ킯
    case osvi.dwPlatformId of

    // Windows NT n.
    VER_PLATFORM_WIN32_NT:
        begin

            // Ɠ肵Ă݂
            if (osvi.dwMajorVersion = 5) and (osvi.dwMinorVersion = 2) then
            begin
                Result := 'Microsoft Windows .NET Server 2003 family, ';
            end else
            if (osvi.dwMajorVersion = 5) and (osvi.dwMinorVersion = 1) then
            begin
                Result := 'Microsoft Windows XP ';
            end else
            if (osvi.dwMajorVersion = 5) and (osvi.dwMinorVersion = 0) then
            begin
                Result := 'Microsoft Windows 2000 ';
            end else
            begin
            if (osvi.dwMajorVersion <= 4) then
            begin
                Result := 'Microsoft Windows NT ';
            end else
                Result := 'Windows NT product family? ';
            end;

            // Windows NT 4.0 SP6 ȍ~Ɠ˂Œׂ.
            if res then
            begin

                // workstation ^Cv
                if (osvi.wProductType = VER_NT_WORKSTATION) then
                begin
                    if (osvi.dwMajorVersion = 4) then
                    begin
                        Result := Result + 'Workstation 4.0 ';
                    end else
                    if (osvi.wSuiteMask and VER_SUITE_PERSONAL) = VER_SUITE_PERSONAL then
                    begin
                        Result := Result + 'Home Edition ';
                    end else
                    begin
                        Result := Result + 'Professional ';
                    end;
                end else

                // server ^Cv
                if (osvi.wProductType = VER_NT_SERVER) then
                begin
                    if (osvi.dwMajorVersion = 5) and (osvi.dwMinorVersion = 2) then
                    begin
                        if (osvi.wSuiteMask and VER_SUITE_DATACENTER) = VER_SUITE_DATACENTER then
                        begin
                            Result := Result + 'Datacenter Edition ';
                        end else
                        if (osvi.wSuiteMask and VER_SUITE_ENTERPRISE) = VER_SUITE_ENTERPRISE then
                        begin
                            Result := Result + 'Enterprise Edition ';
                        end else
                        if (osvi.wSuiteMask = VER_SUITE_BLADE) then
                        begin
                            Result := Result + 'Web Edition ';
                        end else
                        begin
                            Result := Result + 'Standard Edition ';
                        end;
                    end else
                    if (osvi.dwMajorVersion = 5) and (osvi.dwMinorVersion = 0) then
                    begin
                        if (osvi.wSuiteMask and VER_SUITE_DATACENTER) = VER_SUITE_DATACENTER then
                        begin
                            Result := Result + 'Datacenter Server ';
                        end else
                        if (osvi.wSuiteMask and VER_SUITE_ENTERPRISE) = VER_SUITE_ENTERPRISE then
                        begin
                            Result := Result + 'Advanced Server ';
                        end else
                        begin
                            Result := Result + 'Server ';
                        end;
                    end else
                    // Windows NT 4.0
                    begin
                        if (osvi.wSuiteMask and VER_SUITE_ENTERPRISE) = VER_SUITE_ENTERPRISE then
                        begin
                            Result := Result + 'Server 4.0, Enterprise Edition ';
                        end else
                        begin
                            Result := Result + 'Server 4.0 ';
                        end;
                    end;
                end;

            end else
            // Windows NT 4.0 SP5 ȑÔ
            begin
                lRet := RegOpenKeyEx(
                            HKEY_LOCAL_MACHINE,
                            'SYSTEM\CurrentControlSet\Control\ProductOptions',
                            0,
                            KEY_QUERY_VALUE,
                            Key
                        );
                if lRet <> ERROR_SUCCESS then Exit;

                dwBufLen := 255;
                szProductType := '';
                SetLength(szProductType, dwBufLen);

                lRet := RegQueryValueEx(
                            Key,
                            'ProductType',
                            nil,
                            nil,
                            @szProductType[1],
                            @dwBufLen
                        );
                if (lRet <> ERROR_SUCCESS) then Exit;
                dwTemp := Length(szProductType);
                if (dwBufLen > dwTemp) then Exit;
                SetLength(szProductType, dwBufLen);

                RegCloseKey(Key);

                if UpperCase(szProductType) = 'WINNT' then
                begin
                    Result := Result + 'Workstation ';
                end else
                if UpperCase(szProductType) = 'LANMANNT' then
                begin
                    Result := Result + 'Server ';
                end else
                if UpperCase(szProductType) = 'SERVERNT' then
                begin
                    Result := Result + 'Advanced Server ';
                end;

                Result := Result + Format('%d.%d', [osvi.dwMajorVersion, osvi.dwMinorVersion]);
            end;

            // T[rXpbNƂrhԍƂ

            if (osvi.dwMajorVersion = 4) and (lstrcmpi( osvi.szCSDVersion, 'Service Pack 6' ) = 0 ) then
            begin
                // SP6 ? SP6a?

                lRet := RegOpenKeyEx(
                            HKEY_LOCAL_MACHINE,
                            'SOFTWARE\Microsoft\Windows NT\CurrentVersion\Hotfix\Q246009',
                            0,
                            KEY_QUERY_VALUE,
                            Key
                        );
                if (lRet = ERROR_SUCCESS) then
                begin
                    Result := Result + Format('Service Pack 6a (Build %d)', [osvi.dwBuildNumber and $FFFF]);
                end else

                // Windows NT 4.0 prior to SP6a
                begin
                    Result := Result + Format('%s (Build %d)', [osvi.szCSDVersion, osvi.dwBuildNumber and $FFFF]);
                end;
                RegCloseKey(Key);
            end else

            // Windows NT 3.51 and earlier or Windows 2000 and later
            begin
                Result := Result + Format('%s (Build %d)', [osvi.szCSDVersion, osvi.dwBuildNumber and $FFFF]);
            end;

        end;

    // Windows 95 n.
    VER_PLATFORM_WIN32_WINDOWS:
        begin
            // win95
            if (osvi.dwMajorVersion = 4) and (osvi.dwMinorVersion = 0) then
            begin
                Result := 'Microsoft Windows 95 ';
                if (osvi.szCSDVersion[1] = 'C') or (osvi.szCSDVersion[1] = 'B') then
                begin
                    Result := Result + 'OSR2 ';
                end;
            end else
            // win98
            if (osvi.dwMajorVersion = 4) and (osvi.dwMinorVersion = 10) then
            begin
                Result := 'Microsoft Windows 98 ';
                if ( osvi.szCSDVersion[1] = 'A') then
                begin
                    Result := Result + 'SE ';
                end;
            end else
            // winme
            if (osvi.dwMajorVersion = 4) and (osvi.dwMinorVersion = 90) then
            begin
                Result := 'Microsoft Windows Millennium Edition';
            end;
        end;

    // ȊO
    VER_PLATFORM_WIN32s:
        begin
            Result := 'Microsoft Win32s';
        end;

    end;

end;

(* Ăяop *)
function GetOsInfo(): string;
begin
    try
        Result := GetOsInfoHard();
    except
        try
            Result := GetOsInfoEasy();
        except
            Result := 'Microsoft Windows';
        end;
    end;
end;

//=============================================================================
//  oCgLoCg
//  http://hp.vector.co.jp/authors/VA009712/take/delphi/kabesys.htm#getmemory
//=============================================================================
function B2KB(const inByte: Int64): string;
begin
    Result := FormatFloat('#,##0KB', Round(inByte / 1024));
end;

//=============================================================================
//  ̏
//  http://hp.vector.co.jp/authors/VA009712/take/delphi/kabesys.htm#getmemory
//  http://forum.nifty.com/fdelphi/samples/01380.html
//  http://community.borland.com/article/0,1410,16582,00.html
//=============================================================================
procedure GetMemoryStatus(var Phys, Page, Virt: string);
var
    MemoryStatus: TMemoryStatus;
begin
    //  𓾂
    FillChar(MemoryStatus, SizeOf(MemoryStatus), 0);
    MemoryStatus.dwLength := SizeOf(TMemoryStatus);
    GlobalMemoryStatus(MemoryStatus);

    //  o
    with MemoryStatus do
    begin
        Phys := Format('free physical memory:%s/%s',
                        [B2KB(dwAvailPhys), B2KB(dwTotalPhys)]);
        Page := Format('free paging file    :%s/%s',
                        [B2KB(dwAvailPageFile), B2KB(dwTotalPageFile)]);
        Virt := Format('free virtual memory :%s/%s',
                        [B2KB(dwAvailVirtual), B2KB(dwTotalVirtual)]);
    end;
end;

//=============================================================================
//  EXẼo[W
//  http://hp.vector.co.jp/authors/VA026252/tips/delphi_version.html
//=============================================================================
function LoadVersionInfo(const ExePath: string): string;
var
    nSize, nRead: DWORD;
    pBuffer: PChar;
    pValue: PChar;
begin
    Result := '';
    try
        nSize := GetFileVersionInfoSize(PChar(ExePath), nSize);
        if nSize > 0 then
        begin
            pBuffer := AllocMem(nSize);
            try
                GetFileVersionInfo(PChar(ExePath), 0, nSize, pBuffer);
                if VerQueryValue(pBuffer, PChar('StringFileInfo\041103A4\FileVersion'), Pointer(pValue), nRead) then
                begin
                    Result := pValue;
                end;
            finally
                FreeMem(pBuffer, nSize);
            end;
        end;
    except
        ;
    end;
end;

//=============================================================================
//  X^[gAbvtH_
//=============================================================================

//-----------------------------------------------------------------------------
//  X^[gAbvtH_
//  http://hp.vector.co.jp/authors/VA009712/take/delphi/kabesys.htm#ishelllink
function GetStartUpFolderPath(): string;
var
    pidlPath: PItemIDList;
    StartupPath: array[0..MAX_PATH] of Char;
begin
    SHGetSpecialFolderLocation(Application.Handle, CSIDL_STARTUP, pidlPath);
    SHGetPathFromIDList(pidlPath, StartupPath);
    Result := StartupPath;
end;

//-----------------------------------------------------------------------------
//  X^[gAbvɓo^
//  http://hp.vector.co.jp/authors/VA009712/take/delphi/kabesys.htm#ishelllink
procedure CreateStartup();
var
    ShellLink: IShellLink;
    PersistFile: IPersistFile;
    FileName: String;
    LinkFileName: string;
    WFileName: WideString;
    StartupPath: string;
begin
    //  V[gJbg̐ݒ
    FileName := ParamStr(0);
    //  V[gJbgpCOMIuWFNg쐬
    ShellLink := CreateComObject(CLSID_ShellLink) as IShellLink;
    //  V[gJbgt@CpIuWFNg
    PersistFile:= ShellLink as IPersistFile;
    //  t@ČɎw肷p[^
    ShellLink.SetArguments('/starup');
    //  st@C̃tpX
    ShellLink.SetPath(PChar(FileName));
    //  ƃtH_
    ShellLink.SetWorkingDirectory(PChar(ExtractFileDir(FileName)));
    //  X^[gAbvpX̎擾
    StartupPath := GetStartUpFolderPath();
    //  V[gJbgt@C̍쐬
    LinkFileName := ChangeFileExt(ExtractFileName(FileName),'.lnk'); 
    LinkFileName := Format('%s\%s',[StartupPath, LinkFileName]);
    WFileName := WideString(LinkFileName);
    //  t@C쐬
    OleCheck(PersistFile.Save(PWChar(WFileName), False));
end;

//  ---------------------------------------------------------------------------
//  http://homepage1.nifty.com/MADIA/delphi/delphi_bbs/200308_03080004.html

initialization
    CoInitialize(nil);
finalization
    CoUninitialize();
end.
