unit Launcher;

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, IniFiles, Grids, ComCtrls, Menus;

type
    TPlaceboButtonState = record
        Clicking: Boolean;
        PushedCol: integer;
        PushedRow: integer;
    end;

  TLauncherForm = class(TForm)
    GridMain: TStringGrid;
    StatusBar: TStatusBar;
    PopupMenu: TPopupMenu;
    MenuExecute: TMenuItem;
    MenuFolder: TMenuItem;
    Cancel1: TMenuItem;
    N1: TMenuItem;
    N2: TMenuItem;
    MenuCloseForm: TMenuItem;
    MenuQuitApplication: TMenuItem;
    procedure FormCreate(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure FormShow(Sender: TObject);
    procedure GridMainDrawCell(Sender: TObject; ACol, ARow: Integer;
      Rect: TRect; State: TGridDrawState);
    procedure GridMainMouseDown(Sender: TObject; Button: TMouseButton;
      Shift: TShiftState; X, Y: Integer);
    procedure GridMainMouseUp(Sender: TObject; Button: TMouseButton;
      Shift: TShiftState; X, Y: Integer);
    procedure GridMainClick(Sender: TObject);
    procedure MenuExecuteClick(Sender: TObject);
    procedure MenuCloseFormClick(Sender: TObject);
    procedure MenuQuitApplicationClick(Sender: TObject);
    procedure GridMainDblClick(Sender: TObject);
  private
    PreGridProc: TWndMethod;
    PreFormProc: TWndMethod;
    procedure NewGridProc(var Msg: TMessage);
    procedure NewFormProc(var Msg: TMessage);
  private
    FStrBuf: TStringList;
    PBS: TPlaceboButtonState;
    procedure CustomSortPlease(const Col: integer);
  public
    FExeData: TMemIniFile;
  end;

var
  LauncherForm: TLauncherForm;

implementation

uses
    ShellAPI, DateUtils, Math, ExeWatch, Info, DummyMain;

{$R *.dfm}

//=============================================================================
//  JX^\[g֐
//=============================================================================

//  ---------------------------------------------------------------------------
//  tH[ƂȂϐ
var
    MemIniFileForSort: TMemIniFile = nil;
    SortOrder: integer = 1;

//  ---------------------------------------------------------------------------
//  pXŃ\[g
function StringListSortByPath(List: TStringList; Index1, Index2: Integer): Integer;
var
    S1, S2: string;
begin
    //  rΏێo
    S1 := List[Index1];
    S2 := List[Index2];

    //  r
    Result := SortOrder * AnsiCompareText(ExtractFileName(S1), ExtractFileName(S2));
end;

//  ---------------------------------------------------------------------------
//  o[WŃ\[g
function StringListSortByVersion(List: TStringList; Index1, Index2: Integer): Integer;
var
    S1, S2: string;
    SS1, SS2: string;
begin
    //  rΏێo
    S1 := List[Index1];
    S2 := List[Index2];

    //  qf[^o
    SS1 := MemIniFileForSort.ReadString(S1, EXEPROP_VERSION, '');
    SS2 := MemIniFileForSort.ReadString(S2, EXEPROP_VERSION, '');

    //  r
    Result := SortOrder * AnsiCompareText(SS1, SS2);
end;

//  ---------------------------------------------------------------------------
//  ONŃ\[g
function StringListSortByLastBoot(List: TStringList; Index1, Index2: Integer): Integer;
var
    S1, S2: string;
    D1, D2: TDateTime;
begin
    //  rΏێo
    S1 := List[Index1];
    S2 := List[Index2];

    //  qf[^o
    D1 := MemIniFileForSort.ReadDateTime(S1, EXEPROP_LASTBOOT, 0);
    D2 := MemIniFileForSort.ReadDateTime(S2, EXEPROP_LASTBOOT, 0);

    //  r
    Result := SortOrder * CompareDateTime(D1, D2);
end;

//  ---------------------------------------------------------------------------
//  N񐔂Ń\[g
function StringListSortByBootCount(List: TStringList; Index1, Index2: Integer): Integer;
var
    S1, S2: string;
    I1, I2: integer;
begin
    //  rΏێo
    S1 := List[Index1];
    S2 := List[Index2];

    //  qf[^o
    I1 := MemIniFileForSort.ReadInteger(S1, EXEPROP_BOOTCOUNT, 0);
    I2 := MemIniFileForSort.ReadInteger(S2, EXEPROP_BOOTCOUNT, 0);

    //  r
    Result := SortOrder * CompareValue(I1, I2);
end;

//  ---------------------------------------------------------------------------
//  NԂŃ\[g
function StringListSortByBootSum(List: TStringList; Index1, Index2: Integer): Integer;
var
    S1, S2: string;
    I1, I2: integer;
begin
    //  rΏێo
    S1 := List[Index1];
    S2 := List[Index2];

    //  qf[^o
    I1 := MemIniFileForSort.ReadInteger(S1, EXEPROP_BOOTSUM, 0);
    I2 := MemIniFileForSort.ReadInteger(S2, EXEPROP_BOOTSUM, 0);

    //  r
    Result := SortOrder * CompareValue(I1, I2);
end;

//  ---------------------------------------------------------------------------
//  \[g֐ЂƂ܂Ƃ
var
    CustomSortProc: array[0..4] of TStringListSortCompare =
    (
        StringListSortByPath,
        StringListSortByVersion,
        StringListSortByLastBoot,
        StringListSortByBootCount,
        StringListSortByBootSum
    );

//  ---------------------------------------------------------------------------
//  \[gĂяo
procedure TLauncherForm.CustomSortPlease(const Col: integer);
var
    Cell: string;
begin
    MemIniFileForSort := FExeData;
    if (0 < Col) and (Col < GridMain.ColCount) then
    begin
        //  AŌĂяoꍇɏ/~ւ邱Ƃɂ
        Cell := GridMain.Cells[0, Col];

        //  ̗Ƃ̐ȂčlĂ˂
        GridMain.Rows[0].Clear();

        //  O񏸏
        if (Cell = '') then
        begin
            SortOrder := 1;
            GridMain.Cells[0, Col] := '*';
        end
        //  O~
        else
        begin
            SortOrder := -1;
            GridMain.Cells[0, Col] := '';
        end;

        //  \[g
        FStrBuf.CustomSort(CustomSortProc[Col - 1]);
    end;
end;

//=============================================================================
//  tH[\bh
//=============================================================================

//  ---------------------------------------------------------------------------
//  ւEBhE\bh
procedure TLauncherForm.NewGridProc(var Msg: TMessage);
begin
    if (Msg.Msg = WM_ERASEBKGND) then Msg.Result := 0 else PreGridProc(Msg);
end;
//  ---------------------------------------------------------------------------
procedure TLauncherForm.NewFormProc(var Msg: TMessage);
begin
    if (Msg.Msg = WM_ERASEBKGND) then Msg.Result := 0 else PreFormProc(Msg);
end;

//-----------------------------------------------------------------------------
//  tH[쐬
procedure TLauncherForm.FormCreate(Sender: TObject);
var
    i: integer;
begin
    //  Ȃ悤Ȗϑz
    PreGridProc := GridMain.WindowProc;
    GridMain.WindowProc := NewGridProc;
    PreFormProc := Self.WindowProc;
    Self.WindowProc := NewFormProc;

    //  \ݒ
    GridMain.Canvas.Font.Assign(Self.Font);

    //  U{^
    PBS.Clicking  := False;
    PBS.PushedCol := -1;
    PBS.PushedRow := -1;

    //  f[^ǂݏoɎgp
    FStrBuf := TStringList.Create;

    //  Ǘf[^̃XibvVbgRs[Ă邽߂Ɏgp
    FExeData := TMemIniFile.Create(ChangeFileExt(ParamStr(0), '.dummy'));

    //  Obhݒ
    GridMain.Rows[0].CommaText
        := '"@@",'
         + '"PQRSTUVWXO.EXE"' + ','
         + '"9999.9999.9999.9999"' + ','
         + '"9999/99/99@99:99:99"' + ','
         + '"999999[]"' + ','
         + '"999999[b]"';
    for i := 0 to GridMain.ColCount - 1 do
    begin
        GridMain.ColWidths[i] := GridMain.Canvas.TextWidth(GridMain.Cells[i, 0]);
    end;

    //  \[g@
    GridMain.Rows[0].Clear();
end;

//-----------------------------------------------------------------------------
//  tH[j
procedure TLauncherForm.FormDestroy(Sender: TObject);
begin
    FExeData.Free();
    FStrBuf.Free();

    //  Ȃ悤ȖϑzLZ
    GridMain.WindowProc := PreGridProc;
    Self.WindowProc := PreFormProc;
end;

//-----------------------------------------------------------------------------
//  \O
procedure TLauncherForm.FormShow(Sender: TObject);
begin
    //  ĂяoŃXibvVbgԂz

    //  ZNV(EXEpX)ǂݏo
    FExeData.ReadSections(FStrBuf);

    //  \̂߂̍sݒ(+1̓^Cg)
    if (FStrBuf.Count > 0) then
        GridMain.RowCount := FStrBuf.Count + 1
    else
        GridMain.RowCount := 2;
end;

//-----------------------------------------------------------------------------
//  NbN
procedure TLauncherForm.GridMainClick(Sender: TObject);
var
    Grid: TStringGrid;
    Key, Text: string;
begin
    //  LXgĂ
    Grid := TStringGrid(Sender);

    //  f[^H
    if (Grid.Row > FStrBuf.Count) then Exit;

    //  f[^ANZXL[̎o
    Key := FStrBuf.Strings[Grid.Row - 1];
    //  o
    Text := FExeData.ReadString(Key, EXEPROP_DESCRIPT, '');

    //  \
    StatusBar.SimpleText := Text;
end;

//-----------------------------------------------------------------------------
//  _uNbNŋN
procedure TLauncherForm.GridMainDblClick(Sender: TObject);
var
    Grid: TStringGrid;
    Path: string;
begin
    //  LXgĂ
    Grid := TStringGrid(Sender);

    //  f[^H
    if (Grid.Row > FStrBuf.Count) then Exit;

    //  pX̎o
    Path := FStrBuf.Strings[Grid.Row - 1];

    //  s
    SimpleShellExecute(Path);
end;

//-----------------------------------------------------------------------------
//  `掞
//  http://yokohama.cool.ne.jp/chokuto/urawaza/drawicon.html
//  http://www.borland.co.jp/qanda/delphi/d0003222.html
procedure TLauncherForm.GridMainDrawCell(Sender: TObject; ACol,
  ARow: Integer; Rect: TRect; State: TGridDrawState);
const
    GRID_TITLE: array[0..5] of string = ('', 't@C', 'o[W', 'Ogp', 'N', 'N');
var
    Grid: TStringGrid;
    BkBrushColor, BkFontColor: TColor;
    BkFontStyle: TFontStyles;
    OffsetX, OffsetY: integer;
    Rect2: TRect;
    Text, Key: string;
    IsTextCell: Boolean;
    Icon: TIcon;
    IconL, IconS: HICON;
begin
    //  LXgĂ
    Grid := TStringGrid(Sender);
    Text := Grid.Cells[ACol, ARow];

    //  FȂǂobNAbvĂ
    BkBrushColor := Grid.Canvas.Brush.Color;
    BkFontColor  := Grid.Canvas.Font.Color;
    BkFontStyle  := Grid.Canvas.Font.Style;

    //  ʂ̕ZH
    IsTextCell := True;

    //  `悷鎞̈ʒu
    OffsetX := 2;
    OffsetY := (Rect.Bottom - Rect.Top - Grid.Canvas.TextHeight('Wg')) div 2;

    //  `悷̂ɂ傤ǂ悢Ǝv菬͈
    Rect2 := Rect;
    Rect2.Left  := Rect2.Left + OffsetX;
    Rect2.Top   := Rect2.Top + OffsetY;
    Rect2.Right := Rect2.Right - OffsetX;
    Rect2.Bottom := Rect2.Bottom - OffsetY;

    //---------------------------------
    //  ^Cgs
    if (ARow = 0) then
    begin
        //  ^Cg
        Text := GRID_TITLE[ACol];

        //  {^̐Fɂ낦
        Grid.Canvas.Brush.Color := clBtnFace;

        //  NbNH
        if (PBS.Clicking) and (PBS.PushedCol = ACol) and (PBS.PushedRow = ARow) then
        begin
            //  {^
            DrawFrameControl(Grid.Canvas.Handle, Rect, DFC_BUTTON,
                        DFCS_BUTTONPUSH + DFCS_ADJUSTRECT + DFCS_PUSHED);
            //  LvV
            Grid.Canvas.TextOut(Rect2.Left + 1, Rect2.Top + 1, Text);
        end
        else
        begin
            //  {^
            DrawFrameControl(Grid.Canvas.Handle, Rect, DFC_BUTTON,
                        DFCS_BUTTONPUSH + DFCS_ADJUSTRECT);
            //  LvV
            Grid.Canvas.TextOut(Rect2.Left, Rect2.Top, Text);
        end;
    end
    //---------------------------------
    //  f[^s
    else if (ARow <= FStrBuf.Count) then
    begin
        //---------------------------------
        //  Fɂ

        //  IĂ
        if(gdSelected in State) then
        begin
            Grid.Canvas.Brush.Color := clHighlight;
            Grid.Canvas.Font.Color := clHighlightText;
        end
        //  tH[JXĂ
        else if (gdFocused in State) then
        begin
            Grid.Canvas.Brush.Color := clHighlight;
            Grid.Canvas.Font.Color := clHighlightText;
        end
        //  ʂ
        else
        begin
            Grid.Canvas.Brush.Color := clWindow;
        end;

        //---------------------------------
        //  eɂ

        //  f[^ANZXL[̎o
        Key := FStrBuf.Strings[ARow - 1];

        case ACol of

        //  ACR
        0:  begin
                //  wihԂ
                Grid.Canvas.FillRect(Rect);
                //  ACRoĂ݂
                Icon := TIcon.Create();
                try
                    if (ExtractIconEx(PChar(Key), 0, IconL, IconS, 1) > 0) then
                    begin
                        Icon.Handle := IconS;
                        if (Icon.Handle = INVALID_HANDLE_VALUE) then Icon.Handle := IconL;
                        Grid.Canvas.Draw(Rect2.Left, Rect2.Top, Icon);
                    end;
                finally
                    Icon.Free();
                end;
                IsTextCell := False;
            end;

        //  t@C
        1:  begin
                Text := ExtractFileName(Key);
            end;

        //  o[W
        2:  begin
                Text := FExeData.ReadString(Key, EXEPROP_VERSION, '');
            end;

        //  ON
        3:  begin
                Text := FormatDateTime('c', FExeData.ReadDateTime(Key, EXEPROP_LASTBOOT, 0));
            end;

        //  N
        4:  begin
                Text := Format('%5d[]', [FExeData.ReadInteger(Key, EXEPROP_BOOTCOUNT, 0)]);
                Rect2.Left := Rect2.Right - Grid.Canvas.TextWidth(Text);
            end;

        //  N
        5:  begin
                Text := FormatFloat('#,##0[b]', FExeData.ReadInteger(Key, EXEPROP_BOOTSUM, 0) / 1000 );
                Rect2.Left := Rect2.Right - Grid.Canvas.TextWidth(Text);
            end;

        //  ̑
        else
            begin
                ;
            end;

        end;

        if (IsTextCell) then
        begin
            //  wihԂ
            Grid.Canvas.FillRect(Rect);
            //  `悷
            Grid.Canvas.TextOut(Rect2.Left, Rect2.Top, Text);
        end;
    end
    //  f[^Ȃ
    else
    begin
        //  wihԂ
        Grid.Canvas.FillRect(Rect);
    end;

    //  FȂǂXgA
    Grid.Canvas.Brush.Color := BkBrushColor;
    Grid.Canvas.Font.Color  := BkFontColor;
    Grid.Canvas.Font.Style  := BkFontStyle;
end;

//-----------------------------------------------------------------------------
//  {^񂾁H
procedure TLauncherForm.GridMainMouseDown(Sender: TObject;
  Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
var
    Grid: TStringGrid;
    Col, Row: integer;
    R: TRect;
    Path: string;
begin
    //  LXg
    Grid := TStringGrid(Sender);

    //  }EXʒuZʒuɕϊ
    Grid.MouseToCell(X, Y, Col, Row);
    //  wȏꏊȂH
    if (Col < 0) or (Row < 0) and (FStrBuf.Count < Row) then Exit;

    //  }EXʒuZʒuɕϊ
    Grid.MouseToCell(X - 2, Y, Col, Row);
    //  wȏꏊȂH
    if (Col < 0) or (Row < 0) and (FStrBuf.Count < Row) then Exit;

    //  }EXʒuZʒuɕϊ
    Grid.MouseToCell(X + 2, Y, Col, Row);
    //  wȏꏊȂH
    if (Col < 0) or (Row < 0) and (FStrBuf.Count < Row) then Exit;

    //  NbN
    if (Button = mbLeft) then
    begin
        //  Ƀ{^ƌ߂ʒuł͂Ȃ
        if (Row <> 0) or (Col = 0) then Exit;

        //  ܂ꂽ
        PBS.Clicking := True;
        PBS.PushedCol := Col;
        PBS.PushedRow := Row;

        //  ̃Zĕ`
        R := Grid.CellRect(Col, Row);
        InvalidateRect(Grid.Handle, @R, False);
    end
    //  ENbN
    else if (Button = mbRight) then
    begin
        //  {^ł͂Ȃ
        if (Row = 0) then Exit;

        //  s
        Grid.Row := Row;
        Grid.Repaint();

        //  ̍śH
        Path := FStrBuf[Row - 1];

        //  j[\
        MenuExecute.Hint := Path;
        MenuFolder.Hint := ExtractFilePath(Path);
        PopupMenu.Popup(Mouse.CursorPos.X, Mouse.CursorPos.Y);
    end;
end;

//-----------------------------------------------------------------------------
//  {^H
procedure TLauncherForm.GridMainMouseUp(Sender: TObject;
  Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
var
    Grid: TStringGrid;
    Col, Row: integer;
    R: TRect;
    i, Index: integer;
begin
    //  LXg
    Grid := TStringGrid(Sender);

    //  {^łȂΊ֌WȂ
    if (not PBS.Clicking) then Exit;

    //  ɂĂNbNC
    PBS.Clicking := False;

    //  {^̂sĕ`
    for i := 0 to Grid.ColCount - 1 do
    begin
        R := Grid.CellRect(i, 0);
        InvalidateRect(Grid.Handle, @R, False);
    end;

    //  }EXʒuZʒuɕϊ
    Grid.MouseToCell(X, Y, Col, Row);
    //  wȏꏊȂH
    if (Col < 0) or (Row < 0) and (FStrBuf.Count < Row) then Exit;

    //  񂾃ZƓH
    if (PBS.PushedCol <> Col) or (PBS.PushedRow <> Row) then Exit; 

    //  \[g֐̔ԍɌ
    Index := Col - 1;

    //  \[g֐pӂĂ邩H
    if (Index < Low(CustomSortProc)) or (High(CustomSortProc) < Index) then Exit;

    //  NbN
    CustomSortPlease(Col);

    //  ĕ`
    Grid.Repaint();
end;

//-----------------------------------------------------------------------------
//  N
procedure TLauncherForm.MenuExecuteClick(Sender: TObject);
var
    Path: string;
    MenuItem: TMenuItem;
begin
    MenuItem := TMenuItem(Sender);
    Path := MenuItem.Hint;
    SimpleShellExecute(Path);
end;

//-----------------------------------------------------------------------------
//  tH[
procedure TLauncherForm.MenuCloseFormClick(Sender: TObject);
begin
    Self.Close();
end;

//-----------------------------------------------------------------------------
//  AvP[V
procedure TLauncherForm.MenuQuitApplicationClick(Sender: TObject);
begin
    DummyMainForm.Close();
end;

end.

